/* "fcntl.h" -- constants for BSD-style I/O routines (ala VAX C's <file.h>) */
#define O_RDONLY	0
#define O_WRONLY	1
#define O_RDWR		2
#define O_NDELAY	4
#define O_NOWAIT	4
#define O_APPEND	8
#define O_CREAT		0x0200
#define O_TRUNC		0x0400
#define O_EXCL		0x0800
